/**
 * 容器实例
 * Created by litao on 2018/9/29.
 */
import Base from './base/index';
import promise from 'es6-promise';
import Utils from '../utils/index';
import GlobalState from './global-state/index';
import Router from './router/index'
import methods from './methods/index';
import filters from './filters/index';
import directives from './directives/index';
import JBootInstance from './base/instance/index';
import registryComponents from './component/index';
import mockJs from 'mockJs';

import Ajax, { getTokenHook, responseCheckSuccessHook } from './ajax/index';

export { responseCheckSuccessHook };

class Container extends Base{
    constructor (){
        super();

        this.$vue.prototype.$EZV_zIndex = 3000;

        promise.polyfill();

        //初始化自定义封装
        this.initCustomWrap();

        //初始化内置方法等
        this.initInlay();

        //初始化全局JBoot，封装vue实例
        this.initGlobalJBoot();

        //初始化全局状态管理
        let stateModules = super.getConfigure('global-state-modules', Object);
        this.initGlobalState(stateModules);

        //初始化路由实例
        let routerTable = super.getConfigure('router.routerTable', Array, super.fileLoadFunc);

        this.initRouter(routerTable, { before: this.routerBeforeHook.bind(this), after: this.routerAfterHook.bind(this) });

        //初始化国际化实例
    }

    /**
     * 获取vm实例
     * @return {*}
     */
    get vm (){
        return this.$globalState._vm;
    }

    /**
     * 创建vue实例
     * @param el
     * @param root
     */
    createVueInstance (el, root){
        Utils.assert(root, '创建vue实例时，未传入根节点');

        this.beforeCreateInstance();

        return new this.$vue({
            el,
            store: this.$globalState,
            router: this.$routerInstance,
            // i18n: this.initLanguage(),
            render: h => h(root)
        });
    }

    /**
     * 实例创建之前
     */
    beforeCreateInstance (){
        //初始化全局组件
        this.initGlobalComponents();

        //初始化ajax
        this.initAjax();

        //初始化httpApi
        this.initHttpApi();

        //初始化httpApi封装
        this.initHttpWrap();

        //初始化http代理
        this.initHttpProxy();

        //获取http请求的token
        getTokenHook(() => {
            let $token = this.$vue.prototype.$token;
            let tokenKey = $token.getTokenKey();

            if(tokenKey){
                return {
                    tokenKey,
                    tokenValue: $token.get()
                }
            }
        });
    }

    /**
     * 初始化自定义封装
     */
    initCustomWrap (){
        this.initProperty('$wrap', super.getConfigure('custom-wrap', Object));
    }

    /**
     * 初始化内置方法、过滤器、指令等
     */
    initInlay (){
        //初始化方法
        let customMethods = super.getConfigure('custom-methods', Object);
        Object.keys(Object.assign(methods, customMethods)).forEach(key => this.$vue.prototype[key] = methods[key])

        //初始化过滤器
        let customFilters = super.getConfigure('custom-filters', Object);
        Object.keys(Object.assign(filters, customFilters)).forEach(key => this.$vue.filter(key, filters[key]));

        //初始化指令
        let customDirectives = super.getConfigure('custom-directives', Object);
        Object.keys(Object.assign(directives, customDirectives)).forEach(key => this.$vue.directive(key, directives[key]));
    }

    /**
     * 初始化全局JBoot，封装vue实例
     */
    initGlobalJBoot (){
        Utils.equalsType(window.JBoot, undefined) && (window.JBoot = (options) => new JBootInstance(options, this));
    }

    /**
     * 初始化ajax
     */
    initAjax (){
        this.$vue.prototype.$ajax = Ajax;
    }

    /**
     * 初始化httpApi
     */
    initHttpApi (){
        Utils.setProperty(this, '$httpApi', new Map());

        let httpApis = require.context(`sourcePath/`, true, /\.http\.js$/);

        httpApis.keys().forEach(path => {
            let apiName = Utils.extractName(path);
            let httpMethods = httpApis(path);

            const errorMsg = 'httpApi配置异常[' + apiName + '.http.js] 例：\n\n' +
                'export default (http) => { //此处的http为请求实例\n' +
                '   function testApi (){ //api方法名称\n' +
                '       return http({\n' +
                '           url: "/http/test" //请求地址\n ' +
                '           method: "post" //请求方式\n' +
                '           .... //其他参数\n' +
                '       })\n' +
                '   }\n' +
                '   ...  //其他api方法 \n' +
                '}';

            Utils.assert(Utils.equalsType(httpMethods.default, Function), errorMsg);

            httpMethods = httpMethods.default(this.$vue.prototype.$ajax);

            Utils.assert(Utils.equalsType(httpMethods, Object), errorMsg);

            this.$httpApi.set(apiName, httpMethods);

            Object.keys(httpMethods).forEach(m => _console.log('%c注册httpApi: %s', "color: red;", `[${apiName} - ${m}]`));
        });
    }

    /**
     * 初始化httpApi封装
     */
    initHttpWrap (){
        const _self = this;

        this.$vue.prototype.$http = function (moduleName, methodName, data = {}){
            Utils.assert(moduleName, '请传入API模块名称');

            Utils.assert(methodName, '请传入API方法名称');

            let module = _self.$httpApi.get(moduleName);

            Utils.assert(Utils.equalsType(module, Object), `模块[${moduleName}]下不存在httpApi方法`);

            let method = module[methodName];

            Utils.assert(Utils.equalsType(method, Function), `httpApi方法[${moduleName} - ${methodName}]不存在，或不是一个可执行函数`);

            return new Promise((resolve, reject) => {
                method(data).then(resolve).catch(e => {
                    let defaultErrMsg = `httpApi[${moduleName} - ${methodName}]请求异常`;
                    this.$error(e ? (e.message || e.msg || e || defaultErrMsg) : defaultErrMsg);
                    reject(e);
                });
            });
        }
    }

    /**
     * 初始化http代理
     */
    initHttpProxy (){
        let httpProxys = require.context(`sourcePath/`, true, /\.mock\.js$/);

        httpProxys.keys().forEach(path => {
            let proxy = httpProxys(path);

            if(proxy){
                let proxyFileName = Utils.extractName(path);

                const errorMsg = 'httpProxy配置异常[' + proxyFileName + '.mock.js] 例：\n\n' +
                    'export default () => {\n' +
                    '   test:{ //httpProxyName\n' +
                    '       url: /\/http\/test/ //拦截请求的正则\n' +
                    '       method: "post" //拦截的请求方式\n' +
                    '       handler (params){ //params为请求的参数 \n' +
                    '           return {\n' +
                    '               state: 1 //状态标识\n' +
                    '               data: {} //返回值\n' +
                    '               message: "测试成功" //提示信息\n' +
                    '           } //此处为处理后返回给http的值 \n' +
                    '       }\n' +
                    '   }\n' +
                    '   ...  //其他httpProxy方法 \n' +
                    '}';

                Utils.assert(Utils.equalsType(proxy.default, Function), errorMsg);

                let proxyInstance = proxy.default();

                Utils.assert(Utils.equalsType(proxyInstance, Object), errorMsg);

                Object.keys(proxyInstance).forEach(proxyKey => {
                    let proxyObj = proxyInstance[proxyKey];

                    if(proxyObj.url){
                        let method = proxyObj.method || 'get';
                        let handler = proxyObj.handler || function () {
                                _console.warn(`httpProxy[${proxyObj.url}]未找到处理方法，默认返回空对象`);
                                return {};
                            };

                        mockJs.mock(proxyObj.url, method, handler);

                        _console.log('%c注册httpProxy: %s', "color: red;", `url: ${proxyObj.url} ， method: ${method}`);
                    }else{
                        _console.warn(`httpProxy注册异常，实例为：`, proxyObj);
                    }
                });
            }else{
                _console.warn(`httpProxy注册异常，实例为：`, proxy);
            }
        })
    }

    /**
     * 初始化全局组件
     */
    initGlobalComponents (){

        let globalComponents = super.getConfigure('global-components', Object);

        if(Utils.equalsType(globalComponents, Object)){
            function obj(key) {
                return {
                    default: globalComponents[key]
                };
            }
            obj.keys = function (){
                return Object.keys(globalComponents);
            };

            registryComponents(this.$vue, obj);
        }else{
            registryComponents(this.$vue);
        }
    }

    /**
     * 初始化全局状态管理
     */
    initGlobalState (modules){
        this.initProperty('$globalState', new GlobalState(this.$vue, modules));
    }

    /**
     * 初始化路由实例
     * @param routerTable
     * @param hooks
     */
    initRouter (routerTable, hooks){
        const _container = this;

        //设置redirect方法
        let { constant, permission } = routerTable;

        if(constant && Utils.equalsType(constant, Array)){
            constant = handler(constant, constant);
        }

        if(permission && Utils.equalsType(permission, Array)){
            permission = handler(permission, permission);
        }

        function handler(s, sources) {
            return s.map(item => {
                if(item.children && item.children.length > 0){
                    handler(item.children, sources);
                    return Object.assign(item, {
                        redirect: _container.getRedirectPath.bind(_container, sources)
                    });
                }
            });
        }

        //存储原始菜单
        this.$globalState.commit('SAVE_MENU', {
            type: 'menuSource',
            menus: {
                constant,
                permission
            }
        });

        //加载login、404、401、500等页面
        let constantPages = this.generateConstantPages() || [];
        constantPages = constantPages.concat(routerTable.constant || []);
        this.$globalState.commit('SAVE_MENU', {
            type: 'constantPages',
            menus: constantPages
        });

        this.initProperty('$routerInstance', new Router(this.$vue, hooks, constantPages));
    }

    /**
     * 获取重定向路径
     * @param menus
     * @param to
     * @return {*}
     */
    getRedirectPath(menus, to) {
        let instance = this.getRouteInfo(menus, to);
        //如果跟菜单下没有为菜单的子节点，则跳转到没有权限的页面
        if(instance.name === '/' && instance.children.filter(menu => menu.meta && !menu.meta.hidden).length === 0){
            this.vm.$error('您没有当前系统权限，请联系管理员');
            return '/login';
        }else if(instance && instance.children && instance.children.length > 0){
            return instance.path + (instance.path.endsWith('/') ? '' : '/') + instance.children[0].path;
        }
    }

    /**
     * 获取要重定向的route详情
     * @param menus
     * @param to
     * @return {undefined}
     */
    getRouteInfo(menus, to) {
        if(!to) return;

        let matched = to.matched;

        let instance = undefined;

        matched.forEach(item => {
            instance = menus.filter(rs => rs.name === item.name)[0];
            menus = instance.children || [];
        });

        return instance;
    }

    /**
     * 加载login、404、401、500等页面
     */
    generateConstantPages (){
        let constantPages = [];

        let context = require.context(`sourcePath/`, true, /(login\.vue$|400\.vue$|401\.vue$|404\.vue$|500\.vue$|502\.vue$)/);
        context.keys().forEach(constantPageName => {
            let pageName = Utils.extractName(constantPageName);
            constantPages.push({
                name: pageName,
                path: `/${pageName}`,
                component: this.fileLoadFunc(constantPageName.replace('\./', '').replace('\.vue', ''))
            });
        });

        return constantPages;
    }

    /**
     * 路由跳转之前的处理
     * @param to
     * @param from
     * @param next
     */
    routerBeforeHook (to, from, next){
        //执行之前，需要判断是否可以进行权限过滤
        let executePermissionFilter = super.getConfigure('router.executePermissionFilter');

        let successMenus = this.$globalState.getters.menuInfo.successMenus || [];

        if(Utils.equalsType(executePermissionFilter, Function) && executePermissionFilter(this.vm) && successMenus.length === 0){
            this.loadServerMenu().then(() => this.menuPermissionHandler(to, next)).catch(() => this.routerChangeHandler('jumpBefore', to, from, next));
        }else{ //不执行权限过滤器，直接放行
            this.routerChangeHandler('jumpBefore', to, from, next);
        }
    }

    /**
     * 菜单权限处理
     * @param to
     * @param next
     * @return boolean 是否继续执行
     */
    menuPermissionHandler (to, next){
        let successMenus = this.$globalState.getters.menuInfo.successMenus;

        if(this.$globalState.getters.menuInfo.initErrorCode === undefined && (!successMenus || successMenus.length === 0)){
            let menuPermissionConfig = super.getConfigure('router.permissionHook', Function);

            if(menuPermissionConfig && Utils.equalsType(menuPermissionConfig, Function)){
                let menus = this.$globalState.getters.menuInfo.menuSource.permission || [];

                let handlerResult = menus;
                if(menus && menus.length > 0){
                    const _container = this;

                    let serverMenus = this.$globalState.getters.menuInfo.serverMenus;

                    handlerResult = permissionHandler(menus, undefined, serverMenus);

                    function permissionHandler(source, parent, sms) {
                        return source.filter(menu => {
                            if(menuPermissionConfig(_container.vm, menu, parent, sms)){
                                if(menu.children && menu.children.length > 0){
                                    menu.children = permissionHandler(menu.children, menu, sms);
                                }
                                return true;
                            }
                        });
                    }
                }

                if(handlerResult.length === 0){ //没有权限
                    this.$globalState.commit('MENU_INIT_ERROR', 401);
                    next({ name: '401' });
                }else{
                    //可以访问的菜单
                    this.$globalState.commit('SAVE_MENU', {
                        type: 'successMenus',
                        menus: handlerResult
                    });

                    //添加可访问菜单至路由表
                    this.$routerInstance.addRoutes(handlerResult);
                    next(to.path);
                }
            }
        }else{
            return true;
        }
    }

    /**
     * 加载服务器菜单
     */
    async loadServerMenu (){
        let loadServerMenuFunc = super.getConfigure('router.loadServerMenu');

        if(loadServerMenuFunc && Utils.equalsType(loadServerMenuFunc, Function)){
            return new Promise((resolve, reject) => loadServerMenuFunc(this.vm, resolve, reject)).then(serverMenus => {
                //服务器返回的菜单
                this.$globalState.commit('SAVE_MENU', {
                    type: 'serverMenus',
                    menus: serverMenus || []
                });
                return Promise.resolve();
            }).catch(() => Promise.reject());
        }else{
            return Promise.resolve();
        }
    }

    /**
     * 路由跳转之后的处理
     * @param to
     * @param from
     */
    routerAfterHook (to, from){
        let next = (t) => t && this.$routerInstance.push(t);
        let matchedPages = this.$routerInstance.getMatchedComponents(to);

        if(matchedPages.length === 0){
            this.routerChangeHandler('notFound', to, from, next);
        }else{
            this.routerChangeHandler('jumpAfter', to, from, next);
        }
    }

    /**
     * 路由跳转处理
     * @param type 类型 jumpBefore 跳转之前、 jumpAfter 跳转之后、 notFound 未找到页面
     * @param to 要去哪
     * @param from 来自哪
     * @param next 跳转页面 next() 不传参 为继续执行
     */
    routerChangeHandler (type, to, from, next){
        let routerChangeHook = super.getConfigure('router.routerChangeHook', Function);

        if(routerChangeHook && Utils.equalsType(routerChangeHook, Function)){
            routerChangeHook(this.vm, type, to, from, next, this.$globalState);
        }else{
            if(type === 'jumpBefore'){
                next();
            }else if(type === 'notFound'){
                next({ name: '404' });
            }
        }
    }
}

export default Container;