/**
 * Created by litao on 2018/8/10.
 */
import Vue from 'vue';
import cookie from 'js-cookie';
import '../../utils/pwd-string';
import Message from './message/index';

/**
 * 事件传递用
 */
const bus = new Vue();

/**
 * 检测事件名称
 * @param eventName
 * @return {boolean}
 */
function checkEventName(eventName) {
    if (!eventName || Object.prototype.toString.call(eventName).toLowerCase() !== '[object string]') {
        this.$error(`广播事件名[${eventName}]非法`);
        return false;
    }
    return true;
}

export default {
    /**
     * cookie
     */
    $cookie: cookie,

    /**
     * 封装token操作
     */
    $token: {
        get (){
            return window.__JBootInstance__.$cookie.get(this.getTokenKey());
        },

        set (value){
            window.__JBootInstance__.$cookie.set(this.getTokenKey(), value);
        },

        clean (){
            window.__JBootInstance__.$cookie.remove(this.getTokenKey());
        },

        getTokenKey (){
            return window.__JBootInstance__.$tokenKey;
        },

        setTokenKey (value, vm){
            Object.defineProperty(vm.prototype, '$tokenKey', { value })
        }
    },

    /**
     * 获取object对象的原型类型
     * @param obj
     * @return {*}
     */
    $getObjectType(obj) {
        return obj ? Object.prototype.toString.call(obj).toLowerCase() : undefined;
    },

    /**
     * 深度合并
     * @param firstObj
     * @param secondObj
     * @return {{}}
     */
    $deepObjectMerge(firstObj = {}, secondObj = {}) { // 深度合并对象
        for (let key in secondObj) {
            firstObj[key] = firstObj[key] && this.$getObjectType(firstObj[key]) === "[object object]" ?
                this.$getObjectType(secondObj[key]) === '[object function]' ? this.$deepObjectMerge(firstObj[key], secondObj[key]()) :
                    this.$deepObjectMerge(firstObj[key], secondObj[key]) : firstObj[key] = secondObj[key];
        }
        return firstObj;
    },

    /**
     * 深度copy对象
     * @param data
     * @return {*}
     */
    $deepCopy(data) {
        const t = typeOf(data);
        let o;

        if (t === 'array') {
            o = [];
        } else if ( t === 'object') {
            o = {};
        } else {
            return data;
        }

        if (t === 'array') {
            for (let i = 0; i < data.length; i++) {
                o.push($deepCopy(data[i]));
            }
        } else if ( t === 'object') {
            for (let i in data) {
                o[i] = $deepCopy(data[i]);
            }
        }
        return o;
    },

    /**
     * 加密字符数据
     * @param param 加密的数据
     * @param pwd 加密密码
     * @return {{routeParams: *}}
     */
    $encrypt(param, pwd = '') {
        let paramStr = Object.prototype.toString.call(param).toLowerCase() === '[object string]' ? param : JSON.stringify(param);
        return {routeParams: window.pwdString.encrypt(paramStr, pwd)};
    },

    /**
     * 解密字符数据
     * @param str 解密的字符串
     * @param pwd 解密密码
     * @return {*}
     */
    $decrypt(str, pwd) {
        let result = window.pwdString.decrypt(str, pwd);

        result = JSON.parse(result);

        return result;
    },

    /**
     * 根据传入的loopNum循环指定次数
     * @param loopNum 循环次数
     * @param callback 回调
     */
    $loop(loopNum = 0, callback) {
        if (!callback || Object.prototype.toString.call(callback).toLowerCase() !== '[object function]')
            return;

        for (let i = 0; i < loopNum; i++) {
            callback(i);
        }
    },

    /**
     * 事件广播
     * @param eventName
     * @param params
     */
    $busBroadcast(eventName, params) {
        if (checkEventName.call(this, eventName)) {
            bus.$emit(eventName, params);
        }
    },

    /**
     * 监听事件
     * @param eventName
     * @return {Promise}
     */
    $busListener(eventName, callback) {
        if (checkEventName.call(this, eventName) && callback && Object.prototype.toString.call(callback).toLowerCase() === '[object function]') {
            bus.$on(eventName, callback);
        } else {
            reject(`广播事件名[${eventName}]非法`);
        }
    },

    /**
     * 下载
     * @param url
     * @param setting
     * @param params
     */
    $download(url, setting = {}, params) {
        let ajax = new XMLHttpRequest();

        ajax.open(setting.method || 'get', url);

        if (setting.isJson) {
            ajax.setRequestHeader("Content-type", "application/json;charset=UTF-8");
            params = JSON.stringify(params);
        } else {
            ajax.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
            let paramStr = "";

            for (let key in params) {
                if (paramStr) {
                    paramStr += '&';
                }
                let fieldValue = params[key];
                paramStr += key + '=' + fieldValue
            }

            params = paramStr;
        }

        if (setting.headers && setting.headers.length > 0) {
            for (let key in setting.headers) {
                if (setting.headers[key] && setting.headers[key] !== 'null' && setting.headers[key] !== 'undefined') {
                    ajax.setRequestHeader(key, setting.headers[key]);
                }
            }
        }

        ajax.responseType = 'blob';
        ajax.onreadystatechange = function () {
            if (ajax.readyState === 4 && ajax.status === 200) {
                let responseName = ajax.getResponseHeader('download-fileName');
                let filename = responseName || new Date().getTime() + '.xls';
                let blob = new Blob([ajax.response]);
                let csvUrl = URL.createObjectURL(blob);
                let link = document.createElement('a');
                link.href = csvUrl;
                link.download = filename;
                link.click();
            }
        };

        // 判断data send发送数据
        if (params) {
            // 如果有值 从send发送
            ajax.send(params);
        } else {
            // 木有值 直接发送即可
            ajax.send();
        }
    },

    /**
     * 跳转页面
     * @param options
     */
    $jump(options = {name: '404'}) {
        this.$router.push(options);
    },

    /**
     * 获取指定名称slot插槽的实例
     * @param slotName
     * @param self
     * @return {undefined}
     */
    $getSlotInstance(slotName = '', self = this) {
        let slot = self.$getSlot(slotName);
        return slot ? slot.componentInstance : undefined;
    },

    /**
     * 获取指定名称的slot插槽
     * @param slotName
     * @param self
     * @return {*}
     */
    $getSlot(slotName = '', self = this) {
        let result;
        if (slotName) {
            let slots = self.$slots;
            if (slots && slots[slotName] && slots[slotName].length > 0) {
                return slots[slotName][0]
            }
        }
        return result;
    },

    /**
     * 成功
     * @param content
     * @param duration
     */
    $success (content, duration = 3000){
        Message.create({
            inClass: 'animated bounceInDown',
            outClass: 'animated zoomOutRight',
            class: 't-success',
            icon: 'icon-chenggong',
        }, content, duration);
    },

    /**
     * 提示
     * @param content
     * @param duration
     */
    $info (content, duration = 3000){
        Message.create({
            inClass: 'animated bounceInDown',
            outClass: 'animated zoomOutRight',
            class: 't-info',
            icon: 'icon-tishi',
        }, content, duration);
    },

    /**
     * 警告
     * @param content
     * @param duration
     */
    $warn (content, duration = 3000){
        Message.create({
            inClass: 'animated shake',
            outClass: 'animated zoomOutRight',
            class: 't-warn',
            icon: 'icon-jinggao',
        }, content, duration);
    },

    /**
     * 异常
     * @param content
     * @param duration
     */
    $error (content, duration = 3000){
        Message.create({
            inClass: 'animated shake',
            outClass: 'animated zoomOutRight',
            class: 't-error',
            icon: 'icon-shibai',
        }, content, duration);
    },

    /**
     * 确认框
     * @param title
     * @param msg
     * @param cancelFn
     */
    $isOk(title = '提示', msg = '是否继续操作？', cancelFn) {
        return new Promise((resolve, reject) => {
            document.body.appendChild(new Vue({
                render (h){
                    return h('div', {
                        class: 'ev-system-confirm-container',
                        domProps: {
                            id: 'evSystemConfirmContainerId'
                        },
                        style: {
                            zIndex: Vue.prototype.$EZV_zIndex++
                        }
                    }, [
                        h('transition', {
                            props: {
                                name: 'zoomInDown',
                                enterActiveClass: 'animated fadeIn',
                                // leaveActiveClass: 'animated zoomOutDown'
                            }
                        }, [
                            this.show && h('div', {
                                class: 'ev-system-confirm-content',
                                style: {
                                    zIndex: Vue.prototype.$EZV_zIndex++
                                }
                            }, [
                                h('div', {
                                    class: 'ev-system-confirm-content_head'
                                }, [
                                    h('div', {
                                        class: 'ev-system-confirm-content_head-title'
                                    }, title),
                                    h('i', {
                                        class: 'iconfont icon-cuo ev-system-confirm-content_head-close',
                                        on: {
                                            click: () => this.closeEvConfirm(true)
                                        }
                                    })
                                ]),
                                h('div', {
                                    class: 'ev-system-confirm-content_center'
                                }, [
                                    h('i', {
                                        class: 'iconfont icon-wenhaoyuanyiwenmianxing ev-system-confirm-content_center-icon'
                                    }),
                                    h('div', {
                                        class: 'ev-system-confirm-content_center-content'
                                    }, msg)
                                ]),
                                h('div', {
                                    class: 'ev-system-confirm-content_footer'
                                }, [
                                    h('div', {
                                        class: 'ev-system-confirm-content_footer-cancel',
                                        on: {
                                            click: () => this.closeEvConfirm(true)
                                        }
                                    }, '取消'),
                                    h('div', {
                                        class: 'ev-system-confirm-content_footer-confirm',
                                        on: {
                                            click: this.confirmFn
                                        }
                                    }, '确定')
                                ]),
                            ])
                        ])
                    ]);
                },

                data (){
                    return {
                        show: false
                    }
                },

                mounted (){
                    this.show = true;
                },

                methods: {
                    closeEvConfirm (isCancel){
                        let dom = document.getElementById('evSystemConfirmContainerId');
                        document.body.removeChild(dom);
                        isCancel && this.$getObjectType(cancelFn) === '[object function]' && cancelFn();
                    },

                    confirmFn (){
                        this.closeEvConfirm();
                        resolve();
                    }
                }
            }).$mount().$el);
        })
    },

    /**
     * 打开遮罩
     * @param target
     * @param text
     */
    $openMask(target, text = '拼命加载中') {
        if(target){
            this.$closeMask(true);
            let instance;

            if(this.$getObjectType(target) === '[object string]'){ //传进来的是标识
                instance = this.$getRef(target);
                if(instance){
                    instance = instance[0] || instance;
                    if(!instance){
                        instance = document.getElementById(target);
                    }else{
                        instance = instance.$el || instance;
                    }
                }
            }else{
                instance = target.$el || target;
            }

            if(instance && instance instanceof HTMLElement){
                instance.classList.add('ev-system-mask-container_parent');

                instance.appendChild(new Vue({
                    render (h){
                        return h('div', {
                            class: 'ev-system-mask-container',
                            domProps: {
                                id: 'evSystemMaskContainerId',
                            },
                            style: {
                                zIndex: Vue.prototype.$EZV_zIndex++
                            }
                        }, [
                            h('div', {
                                class: 'ev-system-mask-spinner',
                                style: {
                                    zIndex: Vue.prototype.$EZV_zIndex++
                                }
                            }, [
                                h('i', {
                                    class: 'iconfont icon-jiazai ev-system-mask-spinner_icon',
                                    style: {
                                        zIndex: Vue.prototype.$EZV_zIndex++
                                    }
                                }),
                                h('p', {
                                    class: 'ev-system-mask-spinner_text'
                                }, text)
                            ])
                        ]);
                    }
                }).$mount().$el);
            }
        }
    },


    /**
     * 对dom进行平滑滚动
     * @param el 滚动实例
     * @param from 开始值
     * @param to 结束值
     * @param duration 从开始到结束需要的时间 默认500ms
     * @param endCallback 结束之后的回调
     */
    $scrollTop(el, from = 0, to, duration = 500, endCallback) {
        //为保证动画平滑度，避免直接使用timeout
        if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = (
                window.webkitRequestAnimationFrame ||
                window.mozRequestAnimationFrame ||
                window.msRequestAnimationFrame ||
                function (callback) {
                    return window.setTimeout(callback, 1000/60);
                }
            );
        }
        const difference = Math.abs(from - to);
        const step = Math.ceil(difference / duration * 50);

        function scroll(start, end, step) {
            if (start === end) {
                endCallback && endCallback();
                return;
            }

            let d = (start + step > end) ? end : start + step;
            if (start > end) {
                d = (start - step < end) ? end : start - step;
            }

            el.scrollTo(d, d);
            window.requestAnimationFrame(() => scroll(d, end, step));
        }
        scroll(from, to, step);
    },

    /**
     * 关闭遮罩
     * @param isImmediateExecution 是否立即执行 默认延迟执行
     * @return {Promise.<T>}
     */
    $closeMask(isImmediateExecution = false) {
        function closeFn() {
            let maskInstance = document.getElementById('evSystemMaskContainerId');
            if(maskInstance){
                maskInstance.parentElement.removeChild(maskInstance);
            }
        }

        isImmediateExecution ? closeFn() : setTimeout(closeFn);
    },

    /**
     * 获取实例
     * @param ref
     * @param self
     * @return {*}
     */
    $getRef(ref = '', self) {
        if (self) {
            return self.$refs[ref];
        } else {
            return this.$refs[ref];
        }
    },

    /**
     * 退出登录
     * @param sleep
     */
    $logoutHandler (){
        try {
            this.$store.commit('CLEAR_USER');
            this.$store.commit('CLEAR_MENU');
            this.$store.commit('CLEAR_PERMISSION_BUTTONS');
        } catch (e) {

        }
        this.$token.clean();
        this.$jump('/login');
        window.location.reload();
    }
}


/**
 * 日期格式化
 * @param fmt
 * @returns {*}
 */
Date.prototype.format = function (fmt) {
    let o = {
        "M+": this.getMonth() + 1,                 //月份
        "d+": this.getDate(),                    //日
        "h+": this.getHours(),                   //小时
        "m+": this.getMinutes(),                 //分
        "s+": this.getSeconds(),                 //秒
        "q+": Math.floor((this.getMonth() + 3) / 3), //季度
        "S": this.getMilliseconds()             //毫秒
    };
    if (/(y+)/.test(fmt)) {
        fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    }
    for (var k in o) {
        if (new RegExp("(" + k + ")").test(fmt)) {
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
        }
    }
    return fmt;
};

Date.prototype.datetimestr = function () {
    return this.format("yyyy-MM-dd hh:mm:ss");
};

Date.prototype.datestr = function () {
    return this.format("yyyy-MM-dd");
};

Date.prototype.timestr = function () {
    return this.format("hh:mm:ss");
};
if (!Array.prototype.includes) {
    Array.prototype.includes = function (searchStr) {
        if (!searchStr) return false;
        return this.indexOf(searchStr) !== -1;
    };
}

if (!String.prototype.endsWith) {
    String.prototype.endsWith = function (searchStr) {
        let searchIndex = this.indexOf(searchStr);
        return searchIndex === this.length - searchStr.length;
    };
}

if (!Function.prototype.apply) {
    Function.prototype.apply = function (obj, args) {
        obj = obj === undefined ? window : Object(obj);
        let i = 0, ary = [], str;
        if (args) {
            for (len = args.length; i < len; i++) {
                ary[i] = "args[" + i + "]";
            }
        }
        obj._apply = this;
        str = 'obj._apply(' + ary.join(',') + ')';
        try {
            return eval(str);
        } catch (e) {
        } finally {
            delete obj._apply;
        }
    };
}
if (!Function.prototype.call) {
    Function.prototype.call = function (obj) {
        let i = 1, args = [];
        for (let len = arguments.length; i < len; i++) {
            args[i - 1] = arguments[i];
        }
        return this.apply(obj, args);
    };
}